/*
 * Copyright 2013-2018 Amazon.com, Inc. or its affiliates. All Rights Reserved.
 * 
 * Licensed under the Apache License, Version 2.0 (the "License"). You may not use this file except in compliance with
 * the License. A copy of the License is located at
 * 
 * http://aws.amazon.com/apache2.0
 * 
 * or in the "license" file accompanying this file. This file is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR
 * CONDITIONS OF ANY KIND, either express or implied. See the License for the specific language governing permissions
 * and limitations under the License.
 */
package com.amazonaws.services.servicediscovery.model;

import java.io.Serializable;
import javax.annotation.Generated;

import com.amazonaws.AmazonWebServiceRequest;

/**
 * 
 * @see <a href="http://docs.aws.amazon.com/goto/WebAPI/servicediscovery-2017-03-14/DeregisterInstance"
 *      target="_top">AWS API Documentation</a>
 */
@Generated("com.amazonaws:aws-java-sdk-code-generator")
public class DeregisterInstanceRequest extends com.amazonaws.AmazonWebServiceRequest implements Serializable, Cloneable {

    /**
     * <p>
     * The ID of the service that the instance is associated with.
     * </p>
     */
    private String serviceId;
    /**
     * <p>
     * The value that you specified for <code>Id</code> in the <a>RegisterInstance</a> request.
     * </p>
     */
    private String instanceId;

    /**
     * <p>
     * The ID of the service that the instance is associated with.
     * </p>
     * 
     * @param serviceId
     *        The ID of the service that the instance is associated with.
     */

    public void setServiceId(String serviceId) {
        this.serviceId = serviceId;
    }

    /**
     * <p>
     * The ID of the service that the instance is associated with.
     * </p>
     * 
     * @return The ID of the service that the instance is associated with.
     */

    public String getServiceId() {
        return this.serviceId;
    }

    /**
     * <p>
     * The ID of the service that the instance is associated with.
     * </p>
     * 
     * @param serviceId
     *        The ID of the service that the instance is associated with.
     * @return Returns a reference to this object so that method calls can be chained together.
     */

    public DeregisterInstanceRequest withServiceId(String serviceId) {
        setServiceId(serviceId);
        return this;
    }

    /**
     * <p>
     * The value that you specified for <code>Id</code> in the <a>RegisterInstance</a> request.
     * </p>
     * 
     * @param instanceId
     *        The value that you specified for <code>Id</code> in the <a>RegisterInstance</a> request.
     */

    public void setInstanceId(String instanceId) {
        this.instanceId = instanceId;
    }

    /**
     * <p>
     * The value that you specified for <code>Id</code> in the <a>RegisterInstance</a> request.
     * </p>
     * 
     * @return The value that you specified for <code>Id</code> in the <a>RegisterInstance</a> request.
     */

    public String getInstanceId() {
        return this.instanceId;
    }

    /**
     * <p>
     * The value that you specified for <code>Id</code> in the <a>RegisterInstance</a> request.
     * </p>
     * 
     * @param instanceId
     *        The value that you specified for <code>Id</code> in the <a>RegisterInstance</a> request.
     * @return Returns a reference to this object so that method calls can be chained together.
     */

    public DeregisterInstanceRequest withInstanceId(String instanceId) {
        setInstanceId(instanceId);
        return this;
    }

    /**
     * Returns a string representation of this object; useful for testing and debugging.
     *
     * @return A string representation of this object.
     *
     * @see java.lang.Object#toString()
     */
    @Override
    public String toString() {
        StringBuilder sb = new StringBuilder();
        sb.append("{");
        if (getServiceId() != null)
            sb.append("ServiceId: ").append(getServiceId()).append(",");
        if (getInstanceId() != null)
            sb.append("InstanceId: ").append(getInstanceId());
        sb.append("}");
        return sb.toString();
    }

    @Override
    public boolean equals(Object obj) {
        if (this == obj)
            return true;
        if (obj == null)
            return false;

        if (obj instanceof DeregisterInstanceRequest == false)
            return false;
        DeregisterInstanceRequest other = (DeregisterInstanceRequest) obj;
        if (other.getServiceId() == null ^ this.getServiceId() == null)
            return false;
        if (other.getServiceId() != null && other.getServiceId().equals(this.getServiceId()) == false)
            return false;
        if (other.getInstanceId() == null ^ this.getInstanceId() == null)
            return false;
        if (other.getInstanceId() != null && other.getInstanceId().equals(this.getInstanceId()) == false)
            return false;
        return true;
    }

    @Override
    public int hashCode() {
        final int prime = 31;
        int hashCode = 1;

        hashCode = prime * hashCode + ((getServiceId() == null) ? 0 : getServiceId().hashCode());
        hashCode = prime * hashCode + ((getInstanceId() == null) ? 0 : getInstanceId().hashCode());
        return hashCode;
    }

    @Override
    public DeregisterInstanceRequest clone() {
        return (DeregisterInstanceRequest) super.clone();
    }

}
